# -*- coding: utf-8 -*-
"""
Serialization extension for Kombu, the underlying library
used by Celery for asynchronous tasks.

https://docs.celeryproject.org
/projects/kombu/en/stable/userguide/serialization.html

This is used in setuptools to register custom endpoint
"""
from fulfil_client.serialization import dumps, loads, CONTENT_TYPE


register_args = (
    dumps, loads, CONTENT_TYPE, 'utf-8'
)



class Error(Exception):
    def __init__(self, message, code):
        self.message = message
        self.code = code
        super(Exception, self).__init__(message, code)

    def __str__(self):
        return str(self.message)

    def __getnewargs__(self):
        return (self.message, self.code,)


class ServerError(Error):
    """
    Raised when the server returns an internal server error.
    It is unlikely to have any additional information on the
    body as this error was unexpected.
    """

    def __init__(self, message, code, sentry_id=None):
        self.sentry_id = sentry_id
        super(ServerError, self).__init__(message, code)

    def __getnewargs__(self):
        return (self.message, self.code, self.sentry_id)


class ClientError(Error):
    pass


class AuthenticationError(ClientError):
    """
    Failure to properly authenticate yourself in the request.

    This could be because a token expired or becuase the auth
    is just invalid.
    """
    pass


class UserError(ClientError):
    """
    UserError is raised by the Fulfil Server when there
    is a data error or a validation fails. The error messages
    are meant to be user facing and should ideally be displayed
    to the user. User errors generally have a description too, so respect
    that too.
    """
    def __init__(self, message, code, description=None):
        self.description = description
        super(UserError, self).__init__(message, code)

    def __getnewargs__(self):
        return (self.message, self.code, self.description)


class RateLimitError(ClientError):
    pass



# -*- coding: utf-8 -*-
import six
from email.mime.multipart import MIMEMultipart
from email.header import Header
from email.mime.text import MIMEText

if six.PY2:
    from email.MIMEBase import MIMEBase
    from email import Encoders
else:
    from email.mime.base import MIMEBase
    import email.encoders as Encoders


def render_email(
    from_email, to, subject, text_template=None, html_template=None,
    cc=None, attachments=None, **context
):
    """
    Read the templates for email messages, format them, construct
    the email from them and return the corresponding email message
    object.
    :param from_email: Email From
    :param to: Email IDs of direct recepients (list)
    :param subject: Email subject
    :param text_template: String of rendered template with context
        eg in flask: str(render_template(text_template, **context)
    :param html_template: String of transfomed rendered template with context
        eg in flask: str(transform(render_template(html_template, **context)))
        for transform https://premailer.io python library
    :param cc: Email IDs of Cc recepients (list)
    :param attachments: A dict of filename:string as key value pair
                        [preferable file buffer streams]
    :param context: Context to be sent to template rendering
    :return: Email multipart instance or Text/HTML part
    """
    if not (text_template or html_template):
        raise Exception("Atleast HTML or TEXT template is required")

    text_part = None
    if text_template:
        text_part = MIMEText(
            text_template.encode("utf-8"), 'plain', _charset="UTF-8")

    html_part = None
    if html_template:
        html_part = MIMEText(
            html_template.encode("utf-8"), 'html', _charset="UTF-8")

    if text_part and html_part:
        # Construct an alternative part since both the HTML and Text Parts
        # exist.
        message = MIMEMultipart('alternative')
        message.attach(text_part)
        message.attach(html_part)
    else:
        # only one part exists, so use that as the message body.
        message = text_part or html_part

    if attachments:
        # If an attachment exists, the MimeType should be mixed and the
        # message body should just be another part of it.
        message_with_attachments = MIMEMultipart('mixed')

        # Set the message body as the first part
        message_with_attachments.attach(message)

        # Now the message _with_attachments itself becomes the message
        message = message_with_attachments

        for filename, content in attachments.items():
            part = MIMEBase('application', "octet-stream")
            part.set_payload(content)
            Encoders.encode_base64(part)
            # XXX: Filename might have to be encoded with utf-8,
            # i.e., part's encoding or with email's encoding
            part.add_header(
                'Content-Disposition', 'attachment; filename="%s"' % filename
            )
            message.attach(part)

    # If list of addresses are provided for to and cc, then convert it
    # into a string that is "," separated.
    if isinstance(to, (list, tuple)):
        to = ', '.join(to)
    if isinstance(cc, (list, tuple)):
        cc = ', '.join(cc)

    # We need to use Header objects here instead of just assigning the strings
    # in order to get our headers properly encoded (with QP).
    message['Subject'] = Header(subject, 'ISO-8859-1')

    # TODO handle case where domain contains non-ascii letters
    # https://docs.aws.amazon.com/ses/latest/APIReference/API_Destination.html
    message['From'] = from_email
    message['To'] = to
    if cc:
        message['Cc'] = cc

    return message



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
import fulfil_client
from fulfil_client.contrib.mocking import MockFulfil


def api_calling_method():
    client = fulfil_client.Client('apple', 'apples-api-key')
    Product = client.model('product.product')
    products = Product.search_read_all([], None, ['id'])
    Product.write(
        [p['id'] for p in products],
        {'active': False}
    )
    return client


def test_mock_1():
    with MockFulfil('fulfil_client.Client') as mocked_fulfil:
        Product = mocked_fulfil.model('product.product')
        Product.search_read_all.return_value = [
            {'id': 1},
            {'id': 2},
            {'id': 3},
        ]

        # Call the function
        api_calling_method()

        # Now assert
        Product.search_read_all.assert_called()
        Product.search_read_all.assert_called_with([], None, ['id'])
        Product.write.assert_called_with(
            [1, 2, 3], {'active': False}
        )


def test_mock_context():
    "Ensure that old mocks die with the context"
    with MockFulfil('fulfil_client.Client') as mocked_fulfil:
        Product = mocked_fulfil.model('product.product')
        api_calling_method()
        Product.search_read_all.assert_called()

    # Start new context
    with MockFulfil('fulfil_client.Client') as mocked_fulfil:
        Product = mocked_fulfil.model('product.product')
        Product.search_read_all.assert_not_called()


def test_mock_different_return_vals():
    "Return different values based on mock side_effect"
    def lookup_products(domain):
        client = fulfil_client.Client('apple', 'apples-api-key')
        Product = client.model('product.product')
        return Product.search(domain)

    def fake_search(domain):
        # A fake search method that returns ids based on
        # domain.
        if domain == []:
            return [1, 2, 3, 4, 5]
        elif domain == [('salable', '=', True)]:
            return [1, 2, 3]

    with MockFulfil('fulfil_client.Client') as mocked_fulfil:
        Product = mocked_fulfil.model('product.product')
        Product.search.side_effect = fake_search
        assert lookup_products([]) == [1, 2, 3, 4, 5]
        assert lookup_products([('salable', '=', True)]) == [1, 2, 3]



# -*- coding: utf-8 -*-

__author__ = 'Fulfil.IO Inc.'
__email__ = 'hello@fulfil.io'
__version__ = '1.1.0'

# flake8: noqa

from .client import (
    Client, Model, SessionAuth, APIKeyAuth, BearerAuth,
    verify_webhook
)
from .exceptions import ClientError, UserError, ServerError
